@@ -49,6 +49,7 @@ from .rs_helpers import (
     get_memoryview_and_address,
     to_snake_case,
     to_camel_case,
+    device_dropper,
 )
 
 
@@ -56,8 +57,8 @@ logger = logging.getLogger("wgpu")  # noqa
 apidiff = ApiDiff()
 
 # The wgpu-native version that we target/expect
-__version__ = "0.11.0.1"
-__commit_sha__ = "9d962ef667ef6006cca7bac7489d5bf303a2a244"
+__version__ = "0.12.0.1"
+__commit_sha__ = "0ff888a666e6e787af9bc9c9afc35a5055547b5a"
 version_info = tuple(map(int, __version__.split(".")))
 check_expected_version(version_info)  # produces a warning on mismatch
 
@@ -146,18 +147,6 @@ def _tuple_from_tuple_or_dict(ob, fields):
         raise TypeError(error_msg.format(", ".join(fields)))
 
 
-def _loadop_and_clear_from_value(value):
-    """In WebGPU, the load op, can be given either as "load" or a value.
-    The latter translates to "clear" plus that value in wgpu-native.
-    The value can be float/int/color, but we don't deal with that here.
-    """
-    if isinstance(value, str):
-        assert value == "load"
-        return 1, 0  # WGPULoadOp_Load and a stub value
-    else:
-        return 0, value  # WGPULoadOp_Clear and the value
-
-
 _empty_label = ffi.new("char []", b"")
 
 
@@ -413,8 +402,13 @@ class GPUAdapter(base.GPUAdapter):
         label="",
         required_features: "List[enums.FeatureName]" = [],
         required_limits: "Dict[str, int]" = {},
+        default_queue: "structs.QueueDescriptor" = {},
     ):
-        return self._request_device(label, required_features, required_limits, "")
+        if default_queue:
+            check_struct("QueueDescriptor", default_queue)
+        return self._request_device(
+            label, required_features, required_limits, default_queue, ""
+        )
 
     @apidiff.add("a sweet bonus feature from wgpu-native")
     def request_device_tracing(
@@ -424,23 +418,31 @@ class GPUAdapter(base.GPUAdapter):
         label="",
         required_features: "list(enums.FeatureName)" = [],
         required_limits: "Dict[str, int]" = {},
+        default_queue: "structs.QueueDescriptor" = {},
     ):
         """Write a trace of all commands to a file so it can be reproduced
         elsewhere. The trace is cross-platform!
         """
+        if default_queue:
+            check_struct("QueueDescriptor", default_queue)
         if not os.path.isdir(trace_path):
             os.makedirs(trace_path, exist_ok=True)
         elif os.listdir(trace_path):
             logger.warning(f"Trace directory not empty: {trace_path}")
         return self._request_device(
-            label, required_features, required_limits, trace_path
+            label, required_features, required_limits, default_queue, trace_path
         )
 
-    def _request_device(self, label, features, required_limits, trace_path):
+    def _request_device(
+        self, label, features, required_limits, default_queue, trace_path
+    ):
         c_trace_path = ffi.NULL
         if trace_path:  # no-cover
             c_trace_path = ffi.new("char []", trace_path.encode())
 
+        # This is a good moment to drop destroyed devices
+        device_dropper.drop_all_pending()
+
         # Vanilla WGPU does not support interpolating samplers for float32 textures,
         # which is sad for scientific data in particular. We can enable it
         # (on the hardware were wgpu-py likely runs) using the feature:
@@ -468,7 +470,15 @@ class GPUAdapter(base.GPUAdapter):
         for key, val in required_limits.items():
             setattr(c_limits, to_camel_case(key), val)
 
-        # H: nextInChain: WGPUChainedStruct *, label: char *, requiredFeaturesCount: int, requiredFeatures: WGPUFeatureName *, requiredLimits: WGPURequiredLimits *
+        # Note that the struct QueueDescriptor is currently empty :)
+        # H: nextInChain: WGPUChainedStruct *, label: char *
+        queue_struct = new_struct(
+            "WGPUQueueDescriptor",
+            label=to_c_label("default_queue"),
+            # not used: nextInChain
+        )
+
+        # H: nextInChain: WGPUChainedStruct *, label: char *, requiredFeaturesCount: int, requiredFeatures: WGPUFeatureName *, requiredLimits: WGPURequiredLimits *, defaultQueue: WGPUQueueDescriptor
         struct = new_struct_p(
             "WGPUDeviceDescriptor *",
             label=to_c_label(label),
@@ -476,8 +486,8 @@ class GPUAdapter(base.GPUAdapter):
             requiredFeaturesCount=0,
             requiredFeatures=ffi.new("WGPUFeatureName []", []),
             requiredLimits=c_required_limits,
+            defaultQueue=queue_struct,
         )
-
         device_id = None
 
         @ffi.callback("void(WGPURequestDeviceStatus, WGPUDevice, char *, void *)")
@@ -523,9 +533,12 @@ class GPUAdapter(base.GPUAdapter):
         label="",
         required_features: "List[enums.FeatureName]" = [],
         required_limits: "Dict[str, int]" = {},
+        default_queue: "structs.QueueDescriptor" = {},
     ):
+        if default_queue:
+            check_struct("QueueDescriptor", default_queue)
         return self._request_device(
-            label, required_features, required_limits, ""
+            label, required_features, required_limits, default_queue, ""
         )  # no-cover
 
     def _destroy(self):
@@ -535,6 +548,40 @@ class GPUAdapter(base.GPUAdapter):
 
 
 class GPUDevice(base.GPUDevice, GPUObjectBase):
+    def __init__(self, label, internal, adapter, features, limits, queue):
+        super().__init__(label, internal, adapter, features, limits, queue)
+
+        @ffi.callback("void(WGPUErrorType, char *, void *)")
+        def uncaptured_error_callback(c_type, c_message, userdata):
+            error_type = enum_int2str["ErrorType"].get(c_type, "Unknown")
+            message = ffi.string(c_message).decode(errors="ignore")
+            message = message.replace("\\n", "\n")
+            self._on_error(f"Uncaught WGPU error ({error_type}):\n{message}")
+
+        @ffi.callback("void(WGPUDeviceLostReason, char *, void *)")
+        def device_lost_callback(c_reason, c_message, userdata):
+            reason = enum_int2str["DeviceLostReason"].get(c_reason, "Unknown")
+            message = ffi.string(c_message).decode(errors="ignore")
+            message = message.replace("\\n", "\n")
+            self._on_error(f"The WGPU device was lost ({reason}):\n{message}")
+
+        # Keep the refs alive
+        self._uncaptured_error_callback = uncaptured_error_callback
+        self._device_lost_callback = device_lost_callback
+
+        # H: void f(WGPUDevice device, WGPUErrorCallback callback, void * userdata)
+        lib.wgpuDeviceSetUncapturedErrorCallback(
+            self._internal, uncaptured_error_callback, ffi.NULL
+        )
+        # H: void f(WGPUDevice device, WGPUDeviceLostCallback callback, void * userdata)
+        lib.wgpuDeviceSetDeviceLostCallback(
+            self._internal, device_lost_callback, ffi.NULL
+        )
+
+    def _on_error(self, message):
+        """Log the error message (for errors produced by wgpu)."""
+        logger.error(message)
+
     def create_buffer(
         self,
         *,
@@ -597,6 +644,7 @@ class GPUDevice(base.GPUDevice, GPUObjectBase):
         dimension: "enums.TextureDimension" = "2d",
         format: "enums.TextureFormat",
         usage: "flags.TextureUsage",
+        view_formats: "List[enums.TextureFormat]" = [],
     ):
         size = _tuple_from_tuple_or_dict(
             size, ("width", "height", "depth_or_array_layers")
@@ -608,7 +656,13 @@ class GPUDevice(base.GPUDevice, GPUObjectBase):
             height=size[1],
             depthOrArrayLayers=size[2],
         )
-        # H: nextInChain: WGPUChainedStruct *, label: char *, usage: WGPUTextureUsageFlags/int, dimension: WGPUTextureDimension, size: WGPUExtent3D, format: WGPUTextureFormat, mipLevelCount: int, sampleCount: int
+
+        if view_formats:
+            raise NotImplementedError(
+                "create_texture(.. view_formats is not yet supported."
+            )
+
+        # H: nextInChain: WGPUChainedStruct *, label: char *, usage: WGPUTextureUsageFlags/int, dimension: WGPUTextureDimension, size: WGPUExtent3D, format: WGPUTextureFormat, mipLevelCount: int, sampleCount: int, viewFormatCount: int, viewFormats: WGPUTextureFormat *
         struct = new_struct_p(
             "WGPUTextureDescriptor *",
             label=to_c_label(label),
@@ -619,6 +673,8 @@ class GPUDevice(base.GPUDevice, GPUObjectBase):
             format=format,
             usage=usage,
             # not used: nextInChain
+            # not used: viewFormatCount
+            # not used: viewFormats
         )
         # H: WGPUTexture f(WGPUDevice device, WGPUTextureDescriptor const * descriptor)
         id = lib.wgpuDeviceCreateTexture(self._internal, struct)
@@ -642,13 +698,13 @@ class GPUDevice(base.GPUDevice, GPUObjectBase):
         address_mode_w: "enums.AddressMode" = "clamp-to-edge",
         mag_filter: "enums.FilterMode" = "nearest",
         min_filter: "enums.FilterMode" = "nearest",
-        mipmap_filter: "enums.FilterMode" = "nearest",
+        mipmap_filter: "enums.MipmapFilterMode" = "nearest",
         lod_min_clamp: float = 0,
         lod_max_clamp: float = 32,
         compare: "enums.CompareFunction" = None,
         max_anisotropy: int = 1,
     ):
-        # H: nextInChain: WGPUChainedStruct *, label: char *, addressModeU: WGPUAddressMode, addressModeV: WGPUAddressMode, addressModeW: WGPUAddressMode, magFilter: WGPUFilterMode, minFilter: WGPUFilterMode, mipmapFilter: WGPUFilterMode, lodMinClamp: float, lodMaxClamp: float, compare: WGPUCompareFunction, maxAnisotropy: int
+        # H: nextInChain: WGPUChainedStruct *, label: char *, addressModeU: WGPUAddressMode, addressModeV: WGPUAddressMode, addressModeW: WGPUAddressMode, magFilter: WGPUFilterMode, minFilter: WGPUFilterMode, mipmapFilter: WGPUMipmapFilterMode, lodMinClamp: float, lodMaxClamp: float, compare: WGPUCompareFunction, maxAnisotropy: int
         struct = new_struct_p(
             "WGPUSamplerDescriptor *",
             label=to_c_label(label),
@@ -897,12 +953,14 @@ class GPUDevice(base.GPUDevice, GPUObjectBase):
             source_struct[0].chain.next = ffi.NULL
             source_struct[0].chain.sType = lib.WGPUSType_ShaderModuleSPIRVDescriptor
 
-        # H: nextInChain: WGPUChainedStruct *, label: char *
+        # Note, we could give hints here that specify entrypoint and pipelinelayout before compiling
+        # H: nextInChain: WGPUChainedStruct *, label: char *, hintCount: int, hints: WGPUShaderModuleCompilationHint *
         struct = new_struct_p(
             "WGPUShaderModuleDescriptor *",
             label=to_c_label(label),
             nextInChain=ffi.cast("WGPUChainedStruct *", source_struct),
-            # not used: nextInChain
+            hintCount=0,
+            hints=ffi.NULL,
         )
 
         # H: WGPUShaderModule f(WGPUDevice device, WGPUShaderModuleDescriptor const * descriptor)
@@ -1177,7 +1235,6 @@ class GPUDevice(base.GPUDevice, GPUObjectBase):
         id = lib.wgpuDeviceCreateCommandEncoder(self._internal, struct)
         return GPUCommandEncoder(label, id, self)
 
-    # FIXME: new method to implement
     def create_render_bundle_encoder(
         self,
         *,
@@ -1190,15 +1247,13 @@ class GPUDevice(base.GPUDevice, GPUObjectBase):
     ):
         raise NotImplementedError()
 
-    # FIXME: new method to implement
     def create_query_set(self, *, label="", type: "enums.QueryType", count: int):
         raise NotImplementedError()
 
     def _destroy(self):
         if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
-            # H: void f(WGPUDevice device)
-            internal  # lib.wgpuDeviceDrop(internal)  # Causes a hang
+            device_dropper.drop_soon(internal)
 
 
 class GPUBuffer(base.GPUBuffer, GPUObjectBase):
@@ -1421,13 +1476,159 @@ class GPURenderPipeline(base.GPURenderPipeline, GPUPipelineBase, GPUObjectBase):
 
 class GPUCommandBuffer(base.GPUCommandBuffer, GPUObjectBase):
     def _destroy(self):
+        # Since command buffers get destroyed when you submit them, we
+        # must only drop them if they've not been submitted, or we get
+        # 'Cannot remove a vacant resource'. Got this info from the
+        # wgpu chat. Also see
+        # https://docs.rs/wgpu-core/latest/src/wgpu_core/device/mod.rs.html#4180-4194
+        # That's why _internal is set to None in submit()
         if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             # H: void f(WGPUCommandBuffer commandBuffer)
-            internal  # lib.wgpuCommandBufferDrop(internal)  # Causes 'Cannot remove a vacant resource'
+            lib.wgpuCommandBufferDrop(internal)
 
 
-class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
+class GPUCommandsMixin(base.GPUCommandsMixin):
+    pass
+
+
+class GPUBindingCommandsMixin(base.GPUBindingCommandsMixin):
+    def set_bind_group(
+        self,
+        index,
+        bind_group,
+        dynamic_offsets_data,
+        dynamic_offsets_data_start,
+        dynamic_offsets_data_length,
+    ):
+        offsets = list(dynamic_offsets_data)
+        c_offsets = ffi.new("uint32_t []", offsets)
+        bind_group_id = bind_group._internal
+        if isinstance(self, GPUComputePassEncoder):
+            # H: void f(WGPUComputePassEncoder computePassEncoder, uint32_t groupIndex, WGPUBindGroup group, uint32_t dynamicOffsetCount, uint32_t const * dynamicOffsets)
+            lib.wgpuComputePassEncoderSetBindGroup(
+                self._internal, index, bind_group_id, len(offsets), c_offsets
+            )
+        else:
+            # H: void f(WGPURenderPassEncoder renderPassEncoder, uint32_t groupIndex, WGPUBindGroup group, uint32_t dynamicOffsetCount, uint32_t const * dynamicOffsets)
+            lib.wgpuRenderPassEncoderSetBindGroup(
+                self._internal,
+                index,
+                bind_group_id,
+                len(offsets),
+                c_offsets,
+            )
+
+
+class GPUDebugCommandsMixin(base.GPUDebugCommandsMixin):
+    def push_debug_group(self, group_label):
+        c_group_label = ffi.new("char []", group_label.encode())
+        color = 0
+        # todo: these functions are temporarily not available in wgpu-native
+        return  # noqa
+        if isinstance(self, GPUComputePassEncoder):
+            # H: void f(WGPUComputePassEncoder computePassEncoder, char const * groupLabel)
+            lib.wgpuComputePassEncoderPushDebugGroup(
+                self._internal, c_group_label, color
+            )
+        else:
+            # H: void f(WGPURenderPassEncoder renderPassEncoder, char const * groupLabel)
+            lib.wgpuRenderPassEncoderPushDebugGroup(
+                self._internal, c_group_label, color
+            )
+
+    def pop_debug_group(self):
+        # todo: these functions are temporarily not available in wgpu-native
+        return  # noqa
+        if isinstance(self, GPUComputePassEncoder):
+            # H: void f(WGPUComputePassEncoder computePassEncoder)
+            lib.wgpuComputePassEncoderPopDebugGroup(self._internal)
+        else:
+            # H: void f(WGPURenderPassEncoder renderPassEncoder)
+            lib.wgpuRenderPassEncoderPopDebugGroup(self._internal)
+
+    def insert_debug_marker(self, marker_label):
+        c_marker_label = ffi.new("char []", marker_label.encode())
+        color = 0
+        # todo: these functions are temporarily not available in wgpu-native
+        return  # noqa
+        if isinstance(self, GPUComputePassEncoder):
+            # H: void f(WGPUComputePassEncoder computePassEncoder, char const * markerLabel)
+            lib.wgpuComputePassEncoderInsertDebugMarker(
+                self._internal, c_marker_label, color
+            )
+        else:
+            # H: void f(WGPURenderPassEncoder renderPassEncoder, char const * markerLabel)
+            lib.wgpuRenderPassEncoderInsertDebugMarker(
+                self._internal, c_marker_label, color
+            )
+
+
+class GPURenderCommandsMixin(base.GPURenderCommandsMixin):
+    def set_pipeline(self, pipeline):
+        pipeline_id = pipeline._internal
+        # H: void f(WGPURenderPassEncoder renderPassEncoder, WGPURenderPipeline pipeline)
+        lib.wgpuRenderPassEncoderSetPipeline(self._internal, pipeline_id)
+
+    def set_index_buffer(self, buffer, index_format, offset=0, size=None):
+        if not size:
+            size = buffer.size - offset
+        c_index_format = enummap[f"IndexFormat.{index_format}"]
+        # H: void f(WGPURenderPassEncoder renderPassEncoder, WGPUBuffer buffer, WGPUIndexFormat format, uint64_t offset, uint64_t size)
+        lib.wgpuRenderPassEncoderSetIndexBuffer(
+            self._internal, buffer._internal, c_index_format, int(offset), int(size)
+        )
+
+    def set_vertex_buffer(self, slot, buffer, offset=0, size=None):
+        if not size:
+            size = buffer.size - offset
+        # H: void f(WGPURenderPassEncoder renderPassEncoder, uint32_t slot, WGPUBuffer buffer, uint64_t offset, uint64_t size)
+        lib.wgpuRenderPassEncoderSetVertexBuffer(
+            self._internal, int(slot), buffer._internal, int(offset), int(size)
+        )
+
+    def draw(self, vertex_count, instance_count=1, first_vertex=0, first_instance=0):
+        # H: void f(WGPURenderPassEncoder renderPassEncoder, uint32_t vertexCount, uint32_t instanceCount, uint32_t firstVertex, uint32_t firstInstance)
+        lib.wgpuRenderPassEncoderDraw(
+            self._internal, vertex_count, instance_count, first_vertex, first_instance
+        )
+
+    def draw_indirect(self, indirect_buffer, indirect_offset):
+        buffer_id = indirect_buffer._internal
+        # H: void f(WGPURenderPassEncoder renderPassEncoder, WGPUBuffer indirectBuffer, uint64_t indirectOffset)
+        lib.wgpuRenderPassEncoderDrawIndirect(
+            self._internal, buffer_id, int(indirect_offset)
+        )
+
+    def draw_indexed(
+        self,
+        index_count,
+        instance_count=1,
+        first_index=0,
+        base_vertex=0,
+        first_instance=0,
+    ):
+        # H: void f(WGPURenderPassEncoder renderPassEncoder, uint32_t indexCount, uint32_t instanceCount, uint32_t firstIndex, int32_t baseVertex, uint32_t firstInstance)
+        lib.wgpuRenderPassEncoderDrawIndexed(
+            self._internal,
+            index_count,
+            instance_count,
+            first_index,
+            base_vertex,
+            first_instance,
+        )
+
+    def draw_indexed_indirect(self, indirect_buffer, indirect_offset):
+        buffer_id = indirect_buffer._internal
+        # H: void f(WGPURenderPassEncoder renderPassEncoder, WGPUBuffer indirectBuffer, uint64_t indirectOffset)
+        lib.wgpuRenderPassEncoderDrawIndexedIndirect(
+            self._internal, buffer_id, int(indirect_offset)
+        )
+
+
+class GPUCommandEncoder(
+    base.GPUCommandEncoder, GPUCommandsMixin, GPUDebugCommandsMixin, GPUObjectBase
+):
     def begin_compute_pass(
         self,
         *,
@@ -1471,30 +1672,26 @@ class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
                 if color_attachment.get("resolve_target", None) is None
                 else color_attachment["resolve_target"]._internal
             )  # this is a TextureViewId or null
-            c_load_op, clear_color = _loadop_and_clear_from_value(
-                color_attachment["load_value"]
-            )
-            clr = (
-                (0.0, 0.0, 0.0, 0.0)
-                if clear_color == 0
-                else _tuple_from_tuple_or_dict(clear_color, "rgba")
-            )
+            clear_value = color_attachment.get("clear_value", (0, 0, 0, 0))
+            if isinstance(clear_value, dict):
+                check_struct("Color", clear_value)
+                clear_value = _tuple_from_tuple_or_dict("clear_value", "rgba")
             # H: r: float, g: float, b: float, a: float
-            c_clear_color = new_struct(
+            c_clear_value = new_struct(
                 "WGPUColor",
-                r=clr[0],
-                g=clr[1],
-                b=clr[2],
-                a=clr[3],
+                r=clear_value[0],
+                g=clear_value[1],
+                b=clear_value[2],
+                a=clear_value[3],
             )
-            # H: view: WGPUTextureView, resolveTarget: WGPUTextureView, loadOp: WGPULoadOp, storeOp: WGPUStoreOp, clearColor: WGPUColor
+            # H: view: WGPUTextureView, resolveTarget: WGPUTextureView, loadOp: WGPULoadOp, storeOp: WGPUStoreOp, clearValue: WGPUColor
             c_attachment = new_struct(
                 "WGPURenderPassColorAttachment",
                 view=texture_view_id,
                 resolveTarget=c_resolve_target,
-                loadOp=c_load_op,
-                storeOp=color_attachment.get("store_op", "store"),
-                clearColor=c_clear_color,
+                loadOp=color_attachment["load_op"],
+                storeOp=color_attachment["store_op"],
+                clearValue=c_clear_value,
                 # not used: resolveTarget
             )
             c_color_attachments_list.append(c_attachment)
@@ -1505,23 +1702,19 @@ class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
         c_depth_stencil_attachment = ffi.NULL
         if depth_stencil_attachment is not None:
             check_struct("RenderPassDepthStencilAttachment", depth_stencil_attachment)
-            c_depth_load_op, c_depth_clear = _loadop_and_clear_from_value(
-                depth_stencil_attachment["depth_load_value"]
-            )
-            c_stencil_load_op, c_stencil_clear = _loadop_and_clear_from_value(
-                depth_stencil_attachment["stencil_load_value"]
-            )
-            # H: view: WGPUTextureView, depthLoadOp: WGPULoadOp, depthStoreOp: WGPUStoreOp, clearDepth: float, depthReadOnly: bool, stencilLoadOp: WGPULoadOp, stencilStoreOp: WGPUStoreOp, clearStencil: int, stencilReadOnly: bool
+            depth_clear_value = depth_stencil_attachment.get("depth_clear_value", 0)
+            stencil_clear_value = depth_stencil_attachment.get("stencil_clear_value", 0)
+            # H: view: WGPUTextureView, depthLoadOp: WGPULoadOp, depthStoreOp: WGPUStoreOp, depthClearValue: float, depthReadOnly: bool, stencilLoadOp: WGPULoadOp, stencilStoreOp: WGPUStoreOp, stencilClearValue: int, stencilReadOnly: bool
             c_depth_stencil_attachment = new_struct_p(
                 "WGPURenderPassDepthStencilAttachment *",
                 view=depth_stencil_attachment["view"]._internal,
-                depthLoadOp=c_depth_load_op,
+                depthLoadOp=depth_stencil_attachment["depth_load_op"],
                 depthStoreOp=depth_stencil_attachment["depth_store_op"],
-                clearDepth=float(c_depth_clear),
+                depthClearValue=float(depth_clear_value),
                 depthReadOnly=depth_stencil_attachment.get("depth_read_only", False),
-                stencilLoadOp=c_stencil_load_op,
+                stencilLoadOp=depth_stencil_attachment["stencil_load_op"],
                 stencilStoreOp=depth_stencil_attachment["stencil_store_op"],
-                clearStencil=int(c_stencil_clear),
+                stencilClearValue=int(stencil_clear_value),
                 stencilReadOnly=depth_stencil_attachment.get(
                     "stencil_read_only", False
                 ),
@@ -1754,113 +1947,37 @@ class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
         )
         # H: WGPUCommandBuffer f(WGPUCommandEncoder commandEncoder, WGPUCommandBufferDescriptor const * descriptor)
         id = lib.wgpuCommandEncoderFinish(self._internal, struct)
+        # WGPU destroys the command encoder when it's finished. So we set
+        # _internal to None to avoid dropping a nonexistent object.
+        self._internal = None
         return GPUCommandBuffer(label, id, self)
 
-    # FIXME: new method to implement
-    def push_debug_group(self, group_label):
-        raise NotImplementedError()
-
-    # FIXME: new method to implement
-    def pop_debug_group(self):
-        raise NotImplementedError()
-
-    # FIXME: new method to implement
-    def insert_debug_marker(self, marker_label):
-        raise NotImplementedError()
-
-    # FIXME: new method to implement
     def write_timestamp(self, query_set, query_index):
         raise NotImplementedError()
 
-    # FIXME: new method to implement
     def resolve_query_set(
         self, query_set, first_query, query_count, destination, destination_offset
     ):
         raise NotImplementedError()
 
-    def _destroy(self):
-        if self._internal is not None and lib is not None:
-            self._internal, internal = None, self._internal
-            # H: void f(WGPUCommandEncoder commandEncoder)
-            internal  # lib.wgpuCommandEncoderDrop(internal)  # Causes 'Cannot remove a vacant resource'
-
-    # FIXME: new method to implement
     def clear_buffer(self, buffer, offset=0, size=None):
         raise NotImplementedError()
 
-
-class GPUProgrammablePassEncoder(base.GPUProgrammablePassEncoder):
-    def set_bind_group(
-        self,
-        index,
-        bind_group,
-        dynamic_offsets_data,
-        dynamic_offsets_data_start,
-        dynamic_offsets_data_length,
-    ):
-        offsets = list(dynamic_offsets_data)
-        c_offsets = ffi.new("uint32_t []", offsets)
-        bind_group_id = bind_group._internal
-        if isinstance(self, GPUComputePassEncoder):
-            # H: void f(WGPUComputePassEncoder computePassEncoder, uint32_t groupIndex, WGPUBindGroup group, uint32_t dynamicOffsetCount, uint32_t const * dynamicOffsets)
-            lib.wgpuComputePassEncoderSetBindGroup(
-                self._internal, index, bind_group_id, len(offsets), c_offsets
-            )
-        else:
-            # H: void f(WGPURenderPassEncoder renderPassEncoder, uint32_t groupIndex, WGPUBindGroup group, uint32_t dynamicOffsetCount, uint32_t const * dynamicOffsets)
-            lib.wgpuRenderPassEncoderSetBindGroup(
-                self._internal,
-                index,
-                bind_group_id,
-                len(offsets),
-                c_offsets,
-            )
-
-    def push_debug_group(self, group_label):
-        c_group_label = ffi.new("char []", group_label.encode())
-        color = 0
-        # todo: these functions are temporarily not available in wgpu-native
-        return  # noqa
-        if isinstance(self, GPUComputePassEncoder):
-            # H: void f(WGPUComputePassEncoder computePassEncoder, char const * groupLabel)
-            lib.wgpuComputePassEncoderPushDebugGroup(
-                self._internal, c_group_label, color
-            )
-        else:
-            # H: void f(WGPURenderPassEncoder renderPassEncoder, char const * groupLabel)
-            lib.wgpuRenderPassEncoderPushDebugGroup(
-                self._internal, c_group_label, color
-            )
-
-    def pop_debug_group(self):
-        # todo: these functions are temporarily not available in wgpu-native
-        return  # noqa
-        if isinstance(self, GPUComputePassEncoder):
-            # H: void f(WGPUComputePassEncoder computePassEncoder)
-            lib.wgpuComputePassEncoderPopDebugGroup(self._internal)
-        else:
-            # H: void f(WGPURenderPassEncoder renderPassEncoder)
-            lib.wgpuRenderPassEncoderPopDebugGroup(self._internal)
-
-    def insert_debug_marker(self, marker_label):
-        c_marker_label = ffi.new("char []", marker_label.encode())
-        color = 0
-        # todo: these functions are temporarily not available in wgpu-native
-        return  # noqa
-        if isinstance(self, GPUComputePassEncoder):
-            # H: void f(WGPUComputePassEncoder computePassEncoder, char const * markerLabel)
-            lib.wgpuComputePassEncoderInsertDebugMarker(
-                self._internal, c_marker_label, color
-            )
-        else:
-            # H: void f(WGPURenderPassEncoder renderPassEncoder, char const * markerLabel)
-            lib.wgpuRenderPassEncoderInsertDebugMarker(
-                self._internal, c_marker_label, color
-            )
+    def _destroy(self):
+        # Note that the natove object gets destroyed on finish.
+        # Also see GPUCommandBuffer._destroy()
+        if self._internal is not None and lib is not None:
+            self._internal, internal = None, self._internal
+            # H: void f(WGPUCommandEncoder commandEncoder)
+            lib.wgpuCommandEncoderDrop(internal)
 
 
 class GPUComputePassEncoder(
-    base.GPUComputePassEncoder, GPUProgrammablePassEncoder, GPUObjectBase
+    base.GPUComputePassEncoder,
+    GPUCommandsMixin,
+    GPUDebugCommandsMixin,
+    GPUBindingCommandsMixin,
+    GPUObjectBase,
 ):
     """ """
 
@@ -1869,20 +1986,24 @@ class GPUComputePassEncoder(
         # H: void f(WGPUComputePassEncoder computePassEncoder, WGPUComputePipeline pipeline)
         lib.wgpuComputePassEncoderSetPipeline(self._internal, pipeline_id)
 
-    def dispatch(self, x, y=1, z=1):
-        # H: void f(WGPUComputePassEncoder computePassEncoder, uint32_t x, uint32_t y, uint32_t z)
-        lib.wgpuComputePassEncoderDispatch(self._internal, x, y, z)
+    def dispatch_workgroups(
+        self, workgroup_count_x, workgroup_count_y=1, workgroup_count_z=1
+    ):
+        # H: void f(WGPUComputePassEncoder computePassEncoder, uint32_t workgroupCountX, uint32_t workgroupCountY, uint32_t workgroupCountZ)
+        lib.wgpuComputePassEncoderDispatch(
+            self._internal, workgroup_count_x, workgroup_count_y, workgroup_count_z
+        )
 
-    def dispatch_indirect(self, indirect_buffer, indirect_offset):
+    def dispatch_workgroups_indirect(self, indirect_buffer, indirect_offset):
         buffer_id = indirect_buffer._internal
         # H: void f(WGPUComputePassEncoder computePassEncoder, WGPUBuffer indirectBuffer, uint64_t indirectOffset)
         lib.wgpuComputePassEncoderDispatchIndirect(
             self._internal, buffer_id, int(indirect_offset)
         )
 
-    def end_pass(self):
+    def end(self):
         # H: void f(WGPUComputePassEncoder computePassEncoder)
-        lib.wgpuComputePassEncoderEndPass(self._internal)
+        lib.wgpuComputePassEncoderEnd(self._internal)
 
     def _destroy(self):
         if self._internal is not None and lib is not None:
@@ -1890,79 +2011,12 @@ class GPUComputePassEncoder(
             internal  # doesnotexist.wgpuComputePassDrop(internal)
 
 
-class GPURenderEncoderBase(base.GPURenderEncoderBase):
-    """ """
-
-    def set_pipeline(self, pipeline):
-        pipeline_id = pipeline._internal
-        # H: void f(WGPURenderPassEncoder renderPassEncoder, WGPURenderPipeline pipeline)
-        lib.wgpuRenderPassEncoderSetPipeline(self._internal, pipeline_id)
-
-    def set_index_buffer(self, buffer, index_format, offset=0, size=None):
-        if not size:
-            size = buffer.size - offset
-        c_index_format = enummap[f"IndexFormat.{index_format}"]
-        # H: void f(WGPURenderPassEncoder renderPassEncoder, WGPUBuffer buffer, WGPUIndexFormat format, uint64_t offset, uint64_t size)
-        lib.wgpuRenderPassEncoderSetIndexBuffer(
-            self._internal, buffer._internal, c_index_format, int(offset), int(size)
-        )
-
-    def set_vertex_buffer(self, slot, buffer, offset=0, size=None):
-        if not size:
-            size = buffer.size - offset
-        # H: void f(WGPURenderPassEncoder renderPassEncoder, uint32_t slot, WGPUBuffer buffer, uint64_t offset, uint64_t size)
-        lib.wgpuRenderPassEncoderSetVertexBuffer(
-            self._internal, int(slot), buffer._internal, int(offset), int(size)
-        )
-
-    def draw(self, vertex_count, instance_count=1, first_vertex=0, first_instance=0):
-        # H: void f(WGPURenderPassEncoder renderPassEncoder, uint32_t vertexCount, uint32_t instanceCount, uint32_t firstVertex, uint32_t firstInstance)
-        lib.wgpuRenderPassEncoderDraw(
-            self._internal, vertex_count, instance_count, first_vertex, first_instance
-        )
-
-    def draw_indirect(self, indirect_buffer, indirect_offset):
-        buffer_id = indirect_buffer._internal
-        # H: void f(WGPURenderPassEncoder renderPassEncoder, WGPUBuffer indirectBuffer, uint64_t indirectOffset)
-        lib.wgpuRenderPassEncoderDrawIndirect(
-            self._internal, buffer_id, int(indirect_offset)
-        )
-
-    def draw_indexed(
-        self,
-        index_count,
-        instance_count=1,
-        first_index=0,
-        base_vertex=0,
-        first_instance=0,
-    ):
-        # H: void f(WGPURenderPassEncoder renderPassEncoder, uint32_t indexCount, uint32_t instanceCount, uint32_t firstIndex, int32_t baseVertex, uint32_t firstInstance)
-        lib.wgpuRenderPassEncoderDrawIndexed(
-            self._internal,
-            index_count,
-            instance_count,
-            first_index,
-            base_vertex,
-            first_instance,
-        )
-
-    def draw_indexed_indirect(self, indirect_buffer, indirect_offset):
-        buffer_id = indirect_buffer._internal
-        # H: void f(WGPURenderPassEncoder renderPassEncoder, WGPUBuffer indirectBuffer, uint64_t indirectOffset)
-        lib.wgpuRenderPassEncoderDrawIndexedIndirect(
-            self._internal, buffer_id, int(indirect_offset)
-        )
-
-    def _destroy(self):
-        if self._internal is not None and lib is not None:
-            self._internal, internal = None, self._internal
-            internal  # doesnotexist.wgpuRenderPassDrop(internal)
-
-
 class GPURenderPassEncoder(
     base.GPURenderPassEncoder,
-    GPUProgrammablePassEncoder,
-    GPURenderEncoderBase,
+    GPUCommandsMixin,
+    GPUDebugCommandsMixin,
+    GPUBindingCommandsMixin,
+    GPURenderCommandsMixin,
     GPUObjectBase,
 ):
     def set_viewport(self, x, y, width, height, min_depth, max_depth):
@@ -2000,31 +2054,28 @@ class GPURenderPassEncoder(
         # H: void f(WGPURenderPassEncoder renderPassEncoder, uint32_t reference)
         lib.wgpuRenderPassEncoderSetStencilReference(self._internal, int(reference))
 
-    def end_pass(self):
+    def end(self):
         # H: void f(WGPURenderPassEncoder renderPassEncoder)
-        lib.wgpuRenderPassEncoderEndPass(self._internal)
+        lib.wgpuRenderPassEncoderEnd(self._internal)
 
-    # FIXME: new method to implement
     def execute_bundles(self, bundles):
         raise NotImplementedError()
 
-    # FIXME: new method to implement
     def begin_occlusion_query(self, query_index):
         raise NotImplementedError()
 
-    # FIXME: new method to implement
     def end_occlusion_query(self):
         raise NotImplementedError()
 
 
 class GPURenderBundleEncoder(
     base.GPURenderBundleEncoder,
-    GPUProgrammablePassEncoder,
-    GPURenderEncoderBase,
+    GPUCommandsMixin,
+    GPUDebugCommandsMixin,
+    GPUBindingCommandsMixin,
+    GPURenderCommandsMixin,
     GPUObjectBase,
 ):
-
-    # FIXME: new method to implement
     def finish(self, *, label=""):
         raise NotImplementedError()
 
@@ -2035,6 +2086,10 @@ class GPUQueue(base.GPUQueue, GPUObjectBase):
         c_command_buffers = ffi.new("WGPUCommandBuffer []", command_buffer_ids)
         # H: void f(WGPUQueue queue, uint32_t commandCount, WGPUCommandBuffer const * commands)
         lib.wgpuQueueSubmit(self._internal, len(command_buffer_ids), c_command_buffers)
+        # WGPU destroys the resource when submitting. We follow this
+        # to avoid dropping a nonexistent object.
+        for cb in command_buffers:
+            cb._internal = None
 
     def write_buffer(self, buffer, buffer_offset, data, data_offset=0, size=None):
 
@@ -2211,7 +2266,6 @@ class GPUQueue(base.GPUQueue, GPUObjectBase):
 
         return data
 
-    # FIXME: new method to implement
     def on_submitted_work_done(self):
         raise NotImplementedError()
 
