@@ -37,7 +37,7 @@ class WgpuManualOffscreenCanvas(WgpuAutoGui, WgpuOffscreenCanvas):
         return False
 
     def _request_draw(self):
-        pass
+        call_later(0, self.draw)
 
     def present(self, texture_view):
         # This gets called at the end of a draw pass via GPUCanvasContextOffline
@@ -78,5 +78,9 @@ async def mainloop_iter():
 
 def run():
     """Handle all tasks scheduled with call_later and return."""
+    # This runs a stub coroutine. It will also run any pending things
+    # on the loop, like the draw-event scheduled with request_draw. But
+    # it will not handle any *new* events, like the ones scheduled with
+    # calls to request_draw() in the animate function.
     loop = asyncio.get_event_loop_policy().get_event_loop()
     loop.run_until_complete(mainloop_iter())
